deriveValue = [
    {"fieldName": "field1", "lookupVals": {"field2": "FINDTHIS"}, "value": "newvalue"},
    {"fieldName": "field1", "lookupVals": {"field4": "NOFINDTHIS"}, "value": "incorrectvalue"},
    {"fieldName": "field1", "lookupVals": {"field3": "FINDTHIS"}, "value": "correctvalue"},
    {"fieldName": "field100", "lookupVals": {"field101": "ABC"}}
]
