deriveIncludes = [
    {"fieldName": "field1", "deriveFieldName": "field2", "includes": "GOOD,VALUE", "excludes": "BAD,STUFF", "begins": "", "ends": "", "replace": "goodvalue"},
    {"fieldName": "field1", "deriveFieldName": "field2", "includes": "", "excludes": "", "begins": "ABC", "ends": "", "replace": "goodvalue"},
    {"fieldName": "field1", "deriveFieldName": "field2", "includes": "", "excludes": "", "begins": "", "ends": "XYZ", "replace": "goodvalue"},
    {"fieldName": "field100", "deriveFieldName": "field101"}
]
